import glob
import zipfile
import json
EVENT_DICTIONARY_V2 = {"Penalty":0,"Kick-off":1,"Goal":2,"Substitution":3,"Offside":4,"Shots on target":5,
                                "Shots off target":6,"Clearance":7,"Ball out of play":8,"Throw-in":9,"Foul":10,
                                "Indirect free-kick":11,"Direct free-kick":12,"Corner":13,"Yellow card":14
                                ,"Red card":15,"Yellow->red card":16}

INVERSE_EVENT_DICTIONARY_V2 = {0:"Penalty",1:"Kick-off",2:"Goal",3:"Substitution",4:"Offside",5:"Shots on target",
                                6:"Shots off target",7:"Clearance",8:"Ball out of play",9:"Throw-in",10:"Foul",
                                11:"Indirect free-kick",12:"Direct free-kick",13:"Corner",14:"Yellow card"
                                ,15:"Red card",16:"Yellow->red card"}

EVENT_DICTIONARY_V1 = {"card": 0, "subs": 1, "soccer": 2}

INVERSE_EVENT_DICTIONARY_V1 = {0: "card", 1: "subs", 2: "soccer"}


FRAME_CLASS_DICTIONARY = {"Ball":0,
						"Player team left":1,
						"Player team right":2,
						"Goalkeeper team left":3,
						"Goalkeeper team right":4,
						"Main referee":5,
						"Side referee":6,
						"Staff members":7,
						"Player team unknown 1":8,
						"Player team unknown 2":9,
						"Goalkeeper team unknown":10,
						"Goal left post left ":11,
						"Goal left post right":12,
						"Goal left crossbar":13,
						"Goal right post left":14,
						"Goal right post right":15,
						"Goal right crossbar":16,
						"Side line left":17,
						"Side line right":18,
						"Side line top":19,
						"Side line bottom":20,
						"Middle line":21,
						"Big rect. left top":22,
						"Big rect. left bottom":23,
						"Big rect. left main":24,
						"Big rect. right top":25,
						"Big rect. right bottom":26,
						"Big rect. right main":27,
						"Small rect. left top":28,
						"Small rect. left bottom":29,
						"Small rect. left main":30,
						"Small rect. right top":31,
						"Small rect. right bottom":32,
						"Small rect. right main":33,
						"Circle left":34,
						"Circle right":35,
						"Circle central":36,
						"Yellow card":37,
						"Red card":38,
						"Referee flag":39,
						"Wall of players":40,
						"Line unknown":41,
						"Goal unknown":42
}

INVERSE_FRAME_CLASS_DICTIONARY = {0:"Ball",
								1:"Player team left",
								2:"Player team right",
								3:"Goalkeeper team left",
								4:"Goalkeeper team right",
								5:"Main referee",
								6:"Side referee",
								7:"Staff members",
								8:"Player team unknown 1",
								9:"Player team unknown 2",
								10:"Goalkeeper team unknown",
								11:"Goal left post left ",
								12:"Goal left post right",
								13:"Goal left crossbar",
								14:"Goal right post left",
								15:"Goal right post right",
								16:"Goal right crossbar",
								17:"Side line left",
								18:"Side line right",
								19:"Side line top",
								20:"Side line bottom",
								21:"Middle line",
								22:"Big rect. left top",
								23:"Big rect. left bottom",
								24:"Big rect. left main",
								25:"Big rect. right top",
								26:"Big rect. right bottom",
								27:"Big rect. right main",
								28:"Small rect. left top",
								29:"Small rect. left bottom",
								30:"Small rect. left main",
								31:"Small rect. right top",
								32:"Small rect. right bottom",
								33:"Small rect. right main",
								34:"Circle left",
								35:"Circle right",
								36:"Circle central",
								37:"Yellow card",
								38:"Red card",
								39:"Referee flag",
								40:"Wall of players",
								41:"Line unknown",
								42:"Goal unknown"
}

FRAME_CLASS_COLOR_DICTIONARY = {'Ball': '#fb9b0b', 
								'Player team left': '#298af1', 
								'Player team right': '#ed2234', 
								'Goalkeeper team left': '#46d7f5', 
								'Goalkeeper team right': '#e03f91', 
								'Main referee': '#f9f611', 
								'Side referee': '#f3edb7', 
								'Staff members': '#0cf32d', 
								'Player team unknown 1': '#d9ecf6', 
								'Player team unknown 2': '#fadbe6', 
								'Goalkeeper team unknown': '#b7b7b7', 
								'Goal left post left ': '#2b84d3', 
								'Goal left post right': '#d92b2b', 
								'Goal left crossbar': '#16ec47', 
								'Goal right post left': '#f10bbc', 
								'Goal right post right': '#e0f012', 
								'Goal right crossbar': '#df8113', 
								'Side line left': '#15e8b2', 
								'Side line right': '#e60463', 
								'Side line top': '#e3bd17', 
								'Side line bottom': '#9a781e', 
								'Middle line': '#5d2bc3', 
								'Big rect. left top': '#d73b24', 
								'Big rect. left bottom': '#00a2b1', 
								'Big rect. left main': '#919cbf', 
								'Big rect. right top': '#c207e5', 
								'Big rect. right bottom': '#ae565f', 
								'Big rect. right main': '#0024ac', 
								'Small rect. left top': '#a94b94', 
								'Small rect. left bottom': '#cba060', 
								'Small rect. left main': '#c15cc1', 
								'Small rect. right top': '#f07b16', 
								'Small rect. right bottom': '#6427e3', 
								'Small rect. right main': '#ccc8e4', 
								'Circle left': '#0d4372', 
								'Circle right': '#b14f06', 
								'Circle central': '#2096c9', 
								'Yellow card': '#daf21b', 
								'Red card': '#fa150b', 
								'Referee flag': '#d5a263', 
								'Wall of players': '#ddd3b6', 
								'Line unknown': '#980dd2', 
								'Goal unknown': '#3e8bd4'
}



def LoadJsonFromZip(zippedFile, JsonPath):
    with zipfile.ZipFile(zippedFile, "r") as z:
        # print(filename)
        with z.open(JsonPath) as f:
            data = f.read()
            d = json.loads(data.decode("utf-8"))

    return d


class AverageMeter(object):
    """Computes and stores the average and current value"""

    def __init__(self):
        self.reset()

    def reset(self):
        self.val = 0
        self.avg = 0
        self.sum = 0
        self.count = 0

    def update(self, val, n=1):
        self.val = val
        self.sum += val * n
        self.count += n
        self.avg = self.sum / self.count





# import torch
# # import pandas as pd


# # class SoccerNetDataLoaderTorch(torch.utils.data.Dataset):
    
# #     def __init__(self, soccernet_dir, jsonGamesFile=None, transform=None, input="Video_LQ"):
# #         """
# #         Args:
# #             csv_file (string): Path to the csv file with annotations.
# #             root_dir (string): Directory with all the images.
# #             transform (callable, optional): Optional transform to be applied
# #                 on a sample.
# #         """

# #         if jsonGamesFile is None:
# #             jsonGamesFile = Path(__file__).parent / "data/SoccerNetGames.json"

# #         self.jsonGamesFile = jsonGamesFile

# #         with open(jsonGamesFile, "r") as json_file:
# #             dictionary = json.load(json_file)

# #         self.games = []
# #         for championship in dictionary:
# #             for season in dictionary[championship]:
# #                 for game in dictionary[championship][season]:

# #                     self.games.append(os.path.join(championship, season, game))
                    

# #         # self.landmarks_frame = pd.read_csv(csv_file)
# #         self.soccernet_dir = soccernet_dir
# #         self.transform = transform
# #         self.input = input

# #     def __len__(self):
# #         return len(self.games)

# #     def __getitem__(self, idx):
# #         game = self.games[idx]

# #         if self.input == "Video_LQ":
# #             labels = scikit.io.vread(os.path.join(self.games[idx], "Labels.txt")
# #             video1 = scikit.io.vread(os.path.join(self.games[idx], "1.mkv")
# #             video2 = scikit.io.vread(os.path.join(self.games[idx], "2.mkv")

# #             sample={'video1': video1, 'video2': video2,
# #                     'labels': labels}

# #             if self.transform:
# #                 sample = self.transform(sample)

# #         return sample


# import torch
# import torchvision
# import math
# import os
# import argparse
# import numpy as np
# import cv2  # pip install opencv-python (==3.4.11.41)
# import imutils  # pip install imutils
# # import skvideo.io
# from tqdm import tqdm
# import skvideo.io


# def getDuration(video_path):
#     metadata = skvideo.io.ffprobe(video_path)

#     # read "@duration" metadata if availabel
#     try:
#         time_second = float(metadata["video"]["@duration"])
#     except:
#         pass

#     # prioritize "video" "tag" "DURATION" metadata if readily available
#     for entry in metadata["video"]["tag"]:
#         if list(entry.items())[0][1] == "DURATION":
#             # print("entry", entry)
#             duration = list(entry.items())[1][1].split(":")
#             # print(duration)
#             time_second = int(duration[0])*3600 + \
#                 int(duration[1])*60 + float(duration[2])

#     # print("duration video", time_second, video_path)
#     return time_second


# def video2clips(video, stride, clip_len):
#     # nFrames = video.shape[0]
#     start_idx = 0  # math.ceil(-clip_len/2)
#     end_idx = video.shape[0] # - math.ceil(clip_len/2)
#     idx = torch.arange(start=start_idx, end=end_idx, step=stride).long()
#     idxs = []
#     for i in torch.arange(math.ceil(-clip_len/2), math.ceil(clip_len/2)):
#         idxs.append(idx+i)
#     idx = torch.stack(idxs, dim=1)
#     # idx = unfold2(idx, size=clip_len, step=stride)
#     # print(idx)
#     idx = idx.clamp(0, video.shape[0]-1)
#     # print(idx)
#     return video[idx]
#     # i_frames = [for i in range()]
#     # for i in range(nFrames):
#     #     if i

#     #     print(frame[])

# # def resize(vid, size, interpolation='bilinear'):
# #     # NOTE: using bilinear interpolation because we don't work on minibatches
# #     # at this level
# #     scale = None
# #     if isinstance(size, int):
# #         scale = float(size) / min(vid.shape[-2:])
# #         size = None
# #     return torch.nn.functional.interpolate(
# #         vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)


# def crop(vid, i, j, h, w):
#     return vid[..., i:(i + h), j:(j + w)]


# def center_crop(vid, output_size):
#     h, w = vid.shape[-2:]
#     th, tw = output_size

#     i = int(round((h - th) / 2.))
#     j = int(round((w - tw) / 2.))
#     return crop(vid, i, j, th, tw)


# def hflip(vid):
#     return vid.flip(dims=(-1,))


# # NOTE: for those functions, which generally expect mini-batches, we keep them
# # as non-minibatch so that they are applied as if they were 4d (thus image).
# # this way, we only apply the transformation in the spatial domain
# def resize(vid, size, interpolation='bilinear'):
#     # NOTE: using bilinear interpolation because we don't work on minibatches
#     # at this level
#     scale = None
#     if isinstance(size, int):
#         scale = float(size) / min(vid.shape[-2:])
#         size = None
#     return torch.nn.functional.interpolate(
#         vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)


# def pad(vid, padding, fill=0, padding_mode="constant"):
#     # NOTE: don't want to pad on temporal dimension, so let as non-batch
#     # (4d) before padding. This works as expected
#     return torch.nn.functional.pad(vid, padding, value=fill, mode=padding_mode)


# def to_normalized_float_tensor(vid):
#     return vid.permute(3, 0, 1, 2).to(torch.float32) / 255


# def normalize(vid, mean, std):
#     shape = (-1,) + (1,) * (vid.dim() - 1)
#     # shape = (1,) * (vid.dim() - 1) + (-1,)
#     # print(vid.shape)
#     mean = torch.as_tensor(mean).reshape(shape)
#     # print(mean.shape)
#     std = torch.as_tensor(std).reshape(shape)
#     # print(std.shape)
#     return (vid - mean) / std


# class FrameDataset(torch.utils.data.Dataset):
#     def __init__(self, video_path, FPS_desired):
#         self.video_path=video_path
#         self.FPS_desired = FPS_desired
#         # self.clip_len = clip_len        

#         # vframes, aframes, info = torchvision.io.read_video(
#         #     video_path, pts_unit='sec')
#         videodata = skvideo.io.FFmpegReader(video_path)
#         (numframe, _, _, _) = videodata.getShape()
#         videodata = skvideo.io.vreader(video_path)
#         frames = []
#         self.duration = getDuration(video_path)
#         self.FPS = numframe/self.duration
#         stride = self.FPS/self.FPS_desired
#         for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
#             # frame = imutils.resize(frame, height=112)
#             if (i_frame % stride < 1):
#                 # print(frame.shape)
#                 frame = imutils.resize(frame, height=224)  # keep aspect ratio
#                 # number of pixel to remove per side
#                 off_side = int((frame.shape[1] - 224)/2)
#                 frame = frame[:, off_side:-off_side, :]
#                 # print(frame.shape)
#                 # clip = resize(clip, (112, 112))
#                 # print("max frame intensity", np.max(frame))
#                 frames.append(frame)
#         frames = np.array(frames)
#         vframes = torch.from_numpy(frames)
#         print(vframes.shape)
#         vframes = vframes.permute([0, 3, 1, 2])


#         print(vframes.shape)
#         # normalize video
#         # print("max vframes intensity", vframes.max())
#         self.clips = vframes/255.0
#         # print("max self.clips intensity", self.clips.max())
#         # self.clips = normalize(vid=self.clips,
#         #                     mean=[0.485, 0.456, 0.406],
#         #                     std=[0.229, 0.224, 0.225])
#         # shape = (1,)*(vframes.dim()-1) + (-1,)
#         # mean = torch.as_tensor([0.43216, 0.394666, 0.37645]).reshape(shape)
#         # std = torch.as_tensor([0.22803, 0.22145, 0.216989]).reshape(shape)
#         # vframes = (vframes/255.0-mean)/std

#         # create clips
#         # self.clips = video2clips(vframes, stride=stride, clip_len=1)
        
#         # permute and resize at need
#         # print(vframes.shape)
#         # self.clips = vframes.permute([0, 3, 1, 2]) #[:,:,0,:,:]
#         # print(self.clips.shape)

#     def __len__(self):
#         return len(self.clips)



#     def __getitem__(self, idx):
#         # transform = torch.nn.functional.interpolate(
#             # vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)
#         clip = self.clips[idx, :, :, :]
#         # print("clip", clip.shape)
#         # clip = clip[:, :, ::2, 87:-87:2]
#         # print(clip.shape)
#         # clip = resize(clip, (224,224))
#         # print(clip.shape)
#         return clip


# class ClipDataset(torch.utils.data.Dataset):
#     def __init__(self, video_path, FPS_desired, clip_len):
#         self.video_path=video_path
#         self.FPS_desired = FPS_desired
#         self.clip_len = clip_len        

#         # vframes, aframes, info = torchvision.io.read_video(
#         #     video_path, pts_unit='sec')
#         videodata = skvideo.io.FFmpegReader(video_path)
#         (numframe, _, _, _) = videodata.getShape()
#         videodata = skvideo.io.vreader(video_path)
#         frames = []
#         for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
#             frame = imutils.resize(frame, height=224)
#             off_side = int((frame.shape[1] - 224)/2)
#             frame = frame[:, off_side:off_side+224, :]
#             frame = imutils.resize(frame, height=112)  # keep aspect ratio
#             # number of pixel to remove per side
#             # clip = resize(clip, (112, 112))
#             # print("max frame intensity", np.max(frame))
#             frames.append(frame)
#         frames = np.array(frames)
#         vframes = torch.from_numpy(frames)

#         self.duration = getDuration(video_path)
#         self.FPS = vframes.shape[0]/self.duration

#         # normalize video
#         vframes = vframes/255.0
#         # vframes = normalize(vid=vframes,
#         #                     mean=[0.43216, 0.394666, 0.37645],
#         #                     std=[0.22803, 0.22145, 0.216989])
#         # shape = (1,)*(vframes.dim()-1) + (-1,)
#         # mean = torch.as_tensor([0.43216, 0.394666, 0.37645]).reshape(shape)
#         # std = torch.as_tensor([0.22803, 0.22145, 0.216989]).reshape(shape)
#         # vframes = (vframes/255.0-mean)/std

#         # create clips
#         stride = self.FPS/self.FPS_desired
#         self.clips = video2clips(
#             vframes, stride=stride, clip_len=self.clip_len)
#         # print(self.clips.shape)
        
#         # permute and resize at need
#         self.clips = self.clips.permute([0, 4, 1, 2, 3])
#         # print(self.clips.shape)

#     def __len__(self):
#         return len(self.clips)



#     def __getitem__(self, idx):
#         # transform = torch.nn.functional.interpolate(
#             # vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)
#         clip = self.clips[idx, ...]
#         # clip = clip[:, :, ::2, 87:-87:2]
#         # print(clip.shape)
#         # clip = resize(clip, (112,112))
#         # print(clip.shape)
#         return clip


# if __name__ == "__main__":
#     dataset = ClipDataset(
#         "/media/giancos/Football/SoccerNet/england_epl/2014-2015/2015-02-21 - 18-00 Chelsea 1 - 1 Burnley/1.mkv", FPS_desired=2,clip_len=16)
#     dataset = FrameDataset(
#         "/media/giancos/Football/SoccerNet/england_epl/2014-2015/2015-02-21 - 18-00 Chelsea 1 - 1 Burnley/1.mkv", FPS_desired=2)



import zipfile
import argparse
import numpy as np
import json

from SoccerNet.Evaluation.utils_calibration import mirror_labels, evaluate_detection_prediction, scale_points


def evaluate(gt_zip, prediction_zip, threshold=10, width=960, height=540, folder=""):
    gt_archive = zipfile.ZipFile(gt_zip, 'r')
    prediction_archive = zipfile.ZipFile(prediction_zip, 'r')
    gt_jsons = gt_archive.namelist()

    accuracies = []
    precisions = []
    recalls = []
    dict_errors = {}
    per_class_confusion_dict = {}
    total_frames = 0
    missed = 0
    for gt_json in gt_jsons:
        split, name = gt_json.split("/")

        if folder == "":
            pred_name = f"extremities_{name}"
        else:
            pred_name = f"{folder}/extremities_{name}"

        # pred_name = f"{split}/extremities_{name}"
        print(pred_name)

        if pred_name not in prediction_archive.namelist():
            accuracies.append(0.)
            precisions.append(0.)
            recalls.append(0.)
            continue
        prediction = prediction_archive.read(pred_name)
        prediction = json.loads(prediction.decode("utf-8"))
        gt = gt_archive.read(gt_json)
        gt = json.loads(gt.decode('utf-8'))

        predictions = scale_points(prediction, width, height)
        line_annotations = scale_points(gt, width, height)

        img_prediction = predictions
        img_groundtruth = line_annotations
        confusion1, per_class_conf1, reproj_errors1 = evaluate_detection_prediction(img_prediction,
                                                                                    img_groundtruth,
                                                                                    threshold)
        confusion2, per_class_conf2, reproj_errors2 = evaluate_detection_prediction(img_prediction,
                                                                                    mirror_labels(
                                                                                        img_groundtruth),
                                                                                    threshold)

        accuracy1, accuracy2 = 0., 0.
        if confusion1.sum() > 0:
            accuracy1 = confusion1[0, 0] / confusion1.sum()

        if confusion2.sum() > 0:
            accuracy2 = confusion2[0, 0] / confusion2.sum()

        if accuracy1 > accuracy2:
            accuracy = accuracy1
            confusion = confusion1
            per_class_conf = per_class_conf1
            reproj_errors = reproj_errors1
        else:
            accuracy = accuracy2
            confusion = confusion2
            per_class_conf = per_class_conf2
            reproj_errors = reproj_errors2

        accuracies.append(accuracy)
        if confusion[0, :].sum() > 0:
            precision = confusion[0, 0] / (confusion[0, :].sum())
            precisions.append(precision)
        if (confusion[0, 0] + confusion[1, 0]) > 0:
            recall = confusion[0, 0] / (confusion[0, 0] + confusion[1, 0])
            recalls.append(recall)

        for line_class, errors in reproj_errors.items():
            if line_class in dict_errors.keys():
                dict_errors[line_class].extend(errors)
            else:
                dict_errors[line_class] = errors

        for line_class, confusion_mat in per_class_conf.items():
            if line_class in per_class_confusion_dict.keys():
                per_class_confusion_dict[line_class] += confusion_mat
            else:
                per_class_confusion_dict[line_class] = confusion_mat
    results = {}
    results["meanRecall"] = np.mean(recalls)
    results["meanPrecision"] = np.mean(precisions)
    results["meanAccuracies"] = np.mean(accuracies)

    for line_class, confusion_mat in per_class_confusion_dict.items():
        class_accuracy = confusion_mat[0, 0] / confusion_mat.sum()
        class_recall = confusion_mat[0, 0] / (confusion_mat[0, 0] + confusion_mat[1, 0])
        class_precision = confusion_mat[0, 0] / (confusion_mat[0, 0] + confusion_mat[0, 1])
        results[f"{line_class}Precision"] = class_precision
        results[f"{line_class}Recall"] = class_recall
        results[f"{line_class}Accuracy"] = class_accuracy
    return results





from pathlib import Path
import json
import os

def getListGames(split="v1", task="spotting"):

    #  if single split, convert into a list
    if not isinstance(split, list):
        split = [split]

    # if an element is "v1", convert to  train/valid/test
    if "all" in split:
        split = ["train", "valid", "test", "challenge"]
        if task == "frames":
            split = ["train", "valid", "test"]
    if "v1" in split:
        split.pop(split.index("v1"))
        split.append("train")
        split.append("valid")
        split.append("test")

    # if task == "spotting":
        
    listgames = []
    # print(split)
    # loop over splits
    for spl in split: 
        if task == "spotting":
            if spl == "train":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesTrain.json"
            elif spl == "valid":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesValid.json"
            elif spl == "test":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesTest.json"
            elif spl == "challenge":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesChallenge.json"

        elif task == "camera-changes":
            if spl == "train":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesTrain.json"
            elif spl == "valid":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesValid.json"
            elif spl == "test":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesTest.json"
            elif spl == "challenge":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesChallenge.json"


        elif task == "frames":
            if spl == "train":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetFramesTrain.json"
            elif spl == "valid":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetFramesValid.json"
            elif spl == "test":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetFramesTest.json"
            elif spl == "challenge":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetFramesChallenge.json"


        with open(jsonGamesFile, "r") as json_file:
            dictionary = json.load(json_file)

        for championship in dictionary:
            for season in dictionary[championship]:
                for game in dictionary[championship][season]:

                    listgames.append(os.path.join(championship, season, game))
                        
    return listgames


if __name__ == "__main__":
    print(len(getListGames(["v1"],task="camera-changes")))




import urllib.request
import os
from tqdm import tqdm
import json
import random
from SoccerNet.utils import getListGames

class MyProgressBar():
    def __init__(self, filename):
        self.pbar = None
        self.filename = filename

    def __call__(self, block_num, block_size, total_size):
        if not self.pbar:
            self.pbar = tqdm(total=total_size, unit='iB', unit_scale=True)
            self.pbar.set_description(f"Downloading {self.filename}...")
            self.pbar.refresh()  # to show immediately the update

        self.pbar.update(block_size)



import uuid
from google_measurement_protocol import event, report

class OwnCloudDownloader():
    def __init__(self, LocalDirectory, OwnCloudServer):
        self.LocalDirectory = LocalDirectory
        self.OwnCloudServer = OwnCloudServer

        self.client_id = uuid.uuid4()

    def downloadFile(self, path_local, path_owncloud, user=None, password=None, verbose=True):
        # return 0: successfully downloaded
        # return 1: HTTPError
        # return 2: unsupported error
        # return 3: file already exist locally
        # return 4: password is None
        # return 5: user is None

        if password is None:
            print(f"password required for {path_local}")
            return 4
        if user is None:
            return 5

        if user is not None or password is not None:  
            # update Password
             
            password_mgr = urllib.request.HTTPPasswordMgrWithDefaultRealm()
            password_mgr.add_password(
                None, self.OwnCloudServer, user, password)
            handler = urllib.request.HTTPBasicAuthHandler(
                password_mgr)
            opener = urllib.request.build_opener(handler)
            urllib.request.install_opener(opener)

        if os.path.exists(path_local): # check existence
            if verbose:
                print(f"{path_local} already exists")
            return 2

        try:
            try:
                os.makedirs(os.path.dirname(path_local), exist_ok=True)
                urllib.request.urlretrieve(
                    path_owncloud, path_local, MyProgressBar(path_local))

            except urllib.error.HTTPError as identifier:
                print(identifier)
                return 1
        except:
            os.remove(path_local)
            raise
            return 2

        # record googleanalytics event
        data = event('download', os.path.basename(path_owncloud))
        report('UA-99166333-3', self.client_id, data)

        return 0


class SoccerNetDownloader(OwnCloudDownloader):
    def __init__(self, LocalDirectory,
                 OwnCloudServer="https://exrcsdrive.kaust.edu.sa/exrcsdrive/public.php/webdav/"):
        super(SoccerNetDownloader, self).__init__(
            LocalDirectory, OwnCloudServer)
        self.password = None

    def downloadDataTask(self, task, split=["train","valid","test","challenge"], verbose=True, password="SoccerNet"):

        if task == "calibration":
            if "train" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "train.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "train.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="UxCNUvxClQ8Hw2R",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/UxCNUvxClQ8Hw2R # user for calibration splits
                                        password=password,
                                        verbose=verbose)
            if "valid" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "valid.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "valid.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="UxCNUvxClQ8Hw2R",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/UxCNUvxClQ8Hw2R # user for calibration splits
                                        password=password,
                                        verbose=verbose)
            if "test" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "test.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "test.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="UxCNUvxClQ8Hw2R",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/UxCNUvxClQ8Hw2R # user for calibration splits
                                        password=password,
                                        verbose=verbose)
            if "challenge" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "challenge.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "challenge.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="UxCNUvxClQ8Hw2R",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/UxCNUvxClQ8Hw2R # user for calibration splits
                                        password=password,
                                        verbose=verbose)
            if "test_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "calibration_test_json.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "calibration_test_json.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="n8J8hetGNT43KLX",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/n8J8hetGNT43KLX # user for calibration splits GT
                                        password=password,
                                        verbose=verbose)
            if "challenge_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "calibration_challenge_json.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "calibration_challenge_json.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="n8J8hetGNT43KLX",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/n8J8hetGNT43KLX # user for calibration splits GT
                                        password=password,
                                        verbose=verbose)

        # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
        # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/H16Jx8AD39RzhFU # user for reid splits GT 
        elif task == "reid":
            if "train" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "train.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "train.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="Ffr8fsJcljh2Ds5",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download train.zip for reid - but train.zip was not uploaded on the server yet!")
            
            if "valid" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "valid.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "valid.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="Ffr8fsJcljh2Ds5",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download valid.zip for reid - but valid.zip was not uploaded on the server yet!")
            
            if "test" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "test.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "test.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="Ffr8fsJcljh2Ds5",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download test.zip for reid - but test.zip was not uploaded on the server yet!")
            
            if "challenge" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "challenge.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "challenge.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="Ffr8fsJcljh2Ds5",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download challenge.zip for reid - but challenge.zip was not uploaded on the server yet!")

            if "test_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "test_bbox_info.json"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "test_bbox_info.json").replace(' ', '%20').replace('\\', '/'),
                                        user="H16Jx8AD39RzhFU",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download test_labels.zip for reid - but test_labels.zip was not uploaded on the server yet! - or check the password")
            
            if "challenge_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "challenge_bbox_info.json"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "challenge_bbox_info.json").replace(' ', '%20').replace('\\', '/'),
                                        user="H16Jx8AD39RzhFU",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/Ffr8fsJcljh2Ds5 # user for reid splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download challenge_labels.zip for reid - but challenge_labels.zip was not uploaded on the server yet! - or check the password")

        # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/o9tzUs2GcuEwcnr # user for tracking splits
        # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/qWNjAzjEI6hezNf # user for tracking splits GT 
        elif task == "tracking":
            if "train" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "train.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "train.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="o9tzUs2GcuEwcnr",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/o9tzUs2GcuEwcnr # user for tracking splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download train.zip for tracking - but train.zip was not uploaded on the server yet!")
                
            if "test" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "test.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "test.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="o9tzUs2GcuEwcnr",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/o9tzUs2GcuEwcnr # user for tracking splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download test.zip for tracking - but test.zip was not uploaded on the server yet!")

            if "challenge" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "challenge.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "challenge.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="o9tzUs2GcuEwcnr",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/o9tzUs2GcuEwcnr # user for tracking splits
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download challenge.zip for tracking - but challenge.zip was not uploaded on the server yet!")

            if "test_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "test_labels.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "test_labels.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="qWNjAzjEI6hezNf",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/qWNjAzjEI6hezNf # user for tracking splits GT 
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download test_labels.zip for tracking - but test_labels.zip was not uploaded on the server yet! - or check the password")

            if "challenge_labels" in split:
                res = self.downloadFile(path_local=os.path.join(self.LocalDirectory, task, "challenge_labels.zip"),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "challenge_labels.zip").replace(' ', '%20').replace('\\', '/'),
                                        user="qWNjAzjEI6hezNf",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/qWNjAzjEI6hezNf # user for tracking splits GT 
                                        password=password,
                                        verbose=verbose)
                if res == 1: #HTTPError
                    print("This function should download challenge_labels.zip for tracking - but challenge_labels.zip was not uploaded on the server yet! - or check the password")

        elif task == "spotting":
            # When downloading with this function, the data is downloaded on the subfolder "spotting"
            self.LocalDirectory = os.path.join(self.LocalDirectory, "spotting")
            self.downloadGames(files=["1_baidu_soccer_embeddings.npy", "2_baidu_soccer_embeddings.npy", "Labels-v2.json"], split=split)
            self.LocalDirectory = os.path.dirname(self.LocalDirectory)

        else:
            print("ERROR Unknown task:", task)

#     from SoccerNet.Downloader import SoccerNetDownloader
# mySoccerNetDownloader = SoccerNetDownloader(LocalDirectory="path/to/SoccerNet")
# mySoccerNetDownloader.download(task="reid", split=["train","valid","test","challenge"])
# mySoccerNetDownloader.download(task="calibration", split=["train","valid","test","challenge"])
# mySoccerNetDownloader.download(task="tracking", split=["train","valid","test","challenge"])

    def downloadVideoHD(self, game, file):

        FileLocal = os.path.join(self.LocalDirectory, game, file)
        FileURL = os.path.join(self.OwnCloudServer, game, file).replace(' ', '%20')
        FileURL = FileURL.replace('\\', '/')
        if game in getListGames("v1"):
            user = "B72R7dTu1tZtIst"
        if game in getListGames("challenge"):
            user = "gJ8gja7V8SLxYBh"
        res = self.downloadFile(path_local=FileLocal,
                                path_owncloud=FileURL,
                                user=user,  # user for video HQ
                                password=self.password)


    def downloadVideo(self, game, file):

        FileLocal = os.path.join(self.LocalDirectory, game, file)
        FileURL = os.path.join(self.OwnCloudServer, game, file).replace(' ', '%20')
        FileURL = FileURL.replace('\\', '/')

        if game in getListGames("v1"):
            user = "6XYClm33IyBkTgl"
        if game in getListGames("challenge"):
            user = "trXNXsW9W04onBh"
        res = self.downloadFile(path_local=FileLocal,
                                path_owncloud=FileURL,
                                user=user,  # user for video
                                password=self.password)
                                    
    def downloadGameIndex(self, index, files=["1.mkv", "2.mkv", "Labels.json"], verbose=True):
        return self.downloadGame(getListGames("all")[index], files=files, verbose=verbose)

    def downloadGame(self, game, files=["1.mkv", "2.mkv", "Labels.json"], verbose=True):

        if game in getListGames("v1"):
            spl = "v1"
        if game in getListGames("challenge"):
            spl = "challenge"

        for file in files:

            GameDirectory = os.path.join(self.LocalDirectory, game)
            FileURL = os.path.join(self.OwnCloudServer, game, file).replace(' ', '%20')
            FileURL = FileURL.replace('\\', '/')

            os.makedirs(GameDirectory, exist_ok=True)

            # 224p Videos
            if file in ["1_224p.mkv", "2_224p.mkv"]:
                res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "video_224p", game, file).replace(' ', '%20').replace('\\', '/'),
                                        user="MKmZigARdGSoaTT",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/MKmZigARdGSoaTT # user for video 224p
                                        password=self.password,
                                        verbose=verbose)

            # 720p Videos
            if file in ["1_720p.mkv", "2_720p.mkv"]:
                res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                        path_owncloud=os.path.join(self.OwnCloudServer, "video_720p", game, file).replace(' ', '%20').replace('\\', '/'),
                                        user="xNGfp1W3wPeVOmQ",  # https://exrcsdrive.kaust.edu.sa/exrcsdrive/index.php/s/xNGfp1W3wPeVOmQ # user for video 720p
                                        password=self.password,
                                        verbose=verbose)

            if spl == "challenge":  # specific buckets for the challenge set

                # LQ Videos
                if file in ["1.mkv", "2.mkv"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="trXNXsW9W04onBh",  # user for video LQ
                                            password=self.password,
                                            verbose=verbose)

                # HQ Videos
                elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="gJ8gja7V8SLxYBh",  # user for video HQ
                                            password=self.password,
                                            verbose=verbose)
                

                # V3 - Challenge does not exist "yet"
                # elif file in ["Frames-v3.zip", "Labels-v3.json"]:
                #     res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                #                             path_owncloud=FileURL,
                #                             user="okteXlk6jmDXNJc",  # shared folder for V3
                #                             password=self.password, 
                #                             verbose=verbose)

                # Labels
                elif "Labels" in file:
                    # file in ["Labels.json", "Labels_v2.json"]:
                    # elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D"]):
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="WUOSnPSYRC1RY13",  # user for Labels
                                            password=self.password,
                                            verbose=verbose)

                # Features
                elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D", "calibration", "player", "field", "boundingbox", ".npy"]):
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="d4nu5rJ6IilF9B0",  # user for Features
                                            password="SoccerNet",
                                            verbose=verbose)

            else:  # bucket for "v1"
                # LQ Videos
                if file in ["1.mkv", "2.mkv"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="6XYClm33IyBkTgl",  # user for video LQ
                                            password=self.password,
                                            verbose=verbose)

                # HQ Videos
                elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="B72R7dTu1tZtIst",  # user for video HQ
                                            password=self.password,
                                            verbose=verbose)

                # V3
                elif file in ["Frames-v3.zip", "Labels-v3.json"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="okteXlk6jmDXNJc",  # shared folder for V3
                                            password=self.password, 
                                            verbose=verbose)
                                            
                # Labels
                elif "Labels" in file:
                    # elif file in ["Labels.json"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="ZDeEfBzCzseRCLA",  # user for Labels
                                            password="SoccerNet",
                                            verbose=verbose)

                # features
                elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D", "calibration", "player", "field", "boundingbox", ".npy"]):
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="9eRjic29XTk0gS9",  # user for Features
                                            password="SoccerNet",
                                            verbose=verbose)


    def downloadGames(self, files=["1.mkv", "2.mkv", "Labels.json"], split=["v1"], task="spotting", verbose=True, randomized=False):

        if not isinstance(split, list):
            split = [split]
        for spl in split:

            gamelist = getListGames(spl, task)
            if randomized:
                gamelist = random.sample(gamelist,len(gamelist))

            for game in gamelist:
                self.downloadGame(game=game, files=files, verbose=verbose)



                    
if __name__ == "__main__":

    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter

    # Load the arguments
    parser = ArgumentParser(description='Test Downloader',
                            formatter_class=ArgumentDefaultsHelpFormatter)

    parser.add_argument('--SoccerNet_path',   required=True,
                        type=str, help='Path to the SoccerNet-V2 dataset folder')
    parser.add_argument('--password',   required=False,
                        type=str, help='Path to the list of games to treat')
    args = parser.parse_args()

    mySoccerNetDownloader = SoccerNetDownloader(args.SoccerNet_path)
    mySoccerNetDownloader.password = args.password
    # mySoccerNetDownloader.downloadGame(game=getListGames("all")[549], files=[
    #                                    "1_HQ.mkv", "2_HQ.mkv", "video.ini", "Labels.json"])
    mySoccerNetDownloader.downloadGameIndex(index=549, files=[
                                       "1_HQ.mkv", "2_HQ.mkv", "video.ini", "Labels.json"])
    # mySoccerNetDownloader = SoccerNetDownloader()
    #     for file in files:

    #         GameDirectory = os.path.join(self.LocalDirectory, game)
    #         FileURL = os.path.join(
    #             self.OwnCloudServer, game, file).replace(' ', '%20')
    #         os.makedirs(GameDirectory, exist_ok=True)

    #         # LQ Videos
    #         if file in ["1.mkv", "2.mkv"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="trXNXsW9W04onBh",  # user for video LQ
    #                                     password=self.password)

    #         # HQ Videos
    #         elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="gJ8gja7V8SLxYBh",  # user for video HQ
    #                                     password=self.password)

    #         # Labels
    #         elif file in ["Labels.json"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="WUOSnPSYRC1RY13",  # user for Labels
    #                                     password=self.password)

    #         # Labels
    #         elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="d4nu5rJ6IilF9B0",  # user for Features
    #                                     password="SoccerNet")



    # def downloadTestGames(self, files=["1.mkv", "2.mkv", "Labels.json"]):

    #     for game in getListTestGames():

    #         # game = os.path.join(championship, season, game)


    #         for file in files:

    #             GameDirectory = os.path.join(self.LocalDirectory, game)
    #             FileURL = os.path.join(
    #                 self.OwnCloudServer, game, file).replace(' ', '%20')
    #             os.makedirs(GameDirectory, exist_ok=True)

    #             # LQ Videos
    #             if file in ["1.mkv", "2.mkv"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="trXNXsW9W04onBh",  # user for video LQ
    #                                         password=self.password)

    #             # HQ Videos
    #             elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="gJ8gja7V8SLxYBh",  # user for video HQ
    #                                         password=self.password)

    #             # Labels
    #             elif file in ["Labels.json"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="WUOSnPSYRC1RY13",  # user for Labels
    #                                         password=self.password)

    #             # Labels
    #             elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="d4nu5rJ6IilF9B0",  # user for Features
    #                                         password="SoccerNet")

